# Extend Xotcl Library for XotclIDE purposes
# All Classes can belong (be defined) by an Component 
# (or can be extendet by Component)
# There is same Default Component "default" for all Classes,
# which does not belong to any Component
# Class and Object are extended with methods for handlig
# additional metainformation as Component and Method Categories

if {$tcl_platform(platform) eq "windows"} {
    set ad [pwd]
    cd $tcl_library
    package require XOTcl
    package require Tk 8.4
    cd $ad
}
if {![info exists xotcl::version]} {
    puts "loading XOTcl"
    package require XOTcl
    puts "loading Tk"
    package require Tk 8.4
}

namespace import -force xotcl::*

Class instproc moveToComponent {app} {
    set oldapp [my getCompObject]
    $oldapp removeClass [self]
    my setMetadata component $app
    set newapp [IDE::Component getCompObjectForName $app]
    $newapp addClass [self]
}


Object instproc inspect {} {
    my basicInspect
}
Object instproc basicInspect {} {
    IDE::ObjectBrowser newBrowser [self]
}
Object instproc halt {{symbol {}}} {
    if {[Object isobject IDE::Debugger]} {
       IDE::Debugger startDebugging $symbol
    }
}

Object instproc getComponentName {} {
    if {[my hasMetadataNotEmpty component]} {
	return [my getMetadata component]
    }  else {
	return default
    }
}
Object instproc getCompObject {} {
    if {[my hasMetadataNotEmpty component]} {
	return [IDE::Component getCompObjectForName [my getMetadata component]]
    }  else {
	return [IDE::Component getCompObjectForName default]
    }
}
Object intproc getDescription {} {
    return [self]::description
}
Object instproc moveToComponent {app} {
    ::set oldapp [my getCompObject]
    $oldapp removeObject [self]
    my setMetadata component $app
    ::set newapp [IDE::Component getCompObjectForName $app]
    $newapp addObject [self]
}

Object instproc printString {} {
    # please overweite it to specify your Object printSring
    # the return shoul short specify the object contens for
    # displaying in object inspector
    return "[self] instance of [[self] info class] [[self] info mixin]"
}
Object instproc getDescription {} {
    if {[Object isobject [self]::description]} {
        return [self]::description
    }
    return
}
Object instproc trackOn {} {
    IDE::MethodTracker registerObjectForTracking [self]
}
Object instproc trackOff {} {
    IDE::MethodTracker unregisterObjectForTracking [self]
}
Class instproc trackOn {} {
    IDE::MethodTracker registerClassForTracking [self]
}
Class instproc trackOff {} {
    IDE::MethodTracker unregisterClassForTracking [self]
}

# Metadata wrapper from old versions
Object instproc metadata args {
    puts "[self] metadata $args"
    if {[llength $args]==2} {
       my setMetadata [lindex $args 0] [lindex $args 1]
    } else {
	puts "calling metadata [self callingclass]>>[self callingproc] by [self callingobject]"
    }
    next
}
Object instproc hasMetadata key {
    my exists _idemeta($key)
}
Object instproc hasMetadataNotEmpty key {
    expr {[my hasMetadata $key] && [my set _idemeta($key)] ne ""}
}
Object instproc getMetadata key {
    my set _idemeta($key)
}
Object instproc setMetadata {key value} {
    my set _idemeta($key) $value
}
Class IDEMetadataAnalyzer
IDEMetadataAnalyzer instproc unknown {args} {
    if {[llength $args]==4 && [lindex $args 1] eq "idemeta" && [Object isobject [lindex $args 0]]} {
        #  $object setMetadata $type $value
	[lindex $args 0] setMetadata [lindex $args 2] [lindex $args 3]
    } else {
	my lappend metaList $args
    }
}
IDEMetadataAnalyzer @

#
# comments (documentation) interface
#
Object instproc hasComment {} {
  IDE::CommentsContainer objectHasComment [self]
}
Object instproc getComment {} {
  IDE::CommentsContainer objectGetComment [self]
}
Object instproc setComment  {text} {
  IDE::CommentsContainer objectSetComment [self] $text
}
Object instproc hasMethodComment {type method} {
  IDE::CommentsContainer objectMethodHasComment [self] $type $method
}
Object instproc getMethodComment {type method} {
  IDE::CommentsContainer objectMethodGetComment [self] $type $method
}
Object instproc setMethodComment {type method text} {
  IDE::CommentsContainer objectMethodSetComment [self] $type $method $text
}

